using System.Reflection;
using Aspects;
using NUnit.Framework;
using ObjectModel;

namespace Tests
{
   [TestFixture]
   public class after_enhancement
   {
      private Order order;

      [SetUp]
      public void SetUp()
      {
         order = new Order();
      }

      [Test]
      public void the_new_field_exists()
      {
         var fieldInfo = order.GetType().GetField("GeneratedFor_Customer_Orders");
         Assert.That(fieldInfo != null);
      }

      [Test]
      public void the_new_field_has_an_attribute()
      {
         var fieldInfo = typeof(Order).GetField("GeneratedFor_Customer_Orders");
         AssertAssociationExists(fieldInfo);
      }

      [Test]
      public void the_existing_property_has_an_attribute()
      {
         var propertyInfo = typeof(Customer).GetProperty("Orders");
         AssertAssociationExists(propertyInfo);
      }

      private void AssertAssociationExists(ICustomAttributeProvider propertyInfo)
      {
         var attributes = propertyInfo.GetCustomAttributes(typeof(AssociationAttribute), false);
         Assert.That(attributes.Length > 0);
         Assert.That(attributes[0] is AssociationAttribute);
      }
   }
}